-- Adjust the following path
Root = path.getabsolute("../GCC_4_7_0")

if (_ACTION == nil) then return end

local ActionsData = {
  ["codelite"] = {["Dir"]="CL"},
  ["codeblocks"] = {["Dir"]="codeblocks"},
  ["vs2005"] = {["Dir"]="vs2005"}, ["vs2008"] = {["Dir"]="vs2008"}, ["vs2010"] = {["Dir"]="vs2010"}
}

if (ActionsData[_ACTION] == nil) then
  error (_ACTION .. " unsupported")
end

local ConfigurationsData = {
  ["Debug"] = {["Dir"] = "Debug", ["Flags"] = {"Symbols"}, ["Defines"] = {}},
  ["Release"] = {["Dir"] = "Release", ["Flags"] = {"Optimize"}, ["Defines"] = {"NDEBUG"}}
}

local LocationDir = path.join(Root, "project/" .. ActionsData[_ACTION].Dir)

local function DefaultConfiguration()
  for config,data in pairs(ConfigurationsData) do
    configuration(config)
      objdir(path.join(Root, "obj/" .. ActionsData[_ACTION].Dir)) -- premake add $(configName)/$(AppName)
      targetdir(path.join(Root, "bin/" .. ActionsData[_ACTION].Dir .. "/" .. data.Dir))
      flags (data.Flags)
      defines(data.Defines)
  end
end

local function simple_project(prefix, file_list)
  for _,f in pairs(file_list) do
    local name = prefix .. path.getbasename(f)
    project(name)
      kind "ConsoleApp"
      language "C++"
      targetname(name)
      files { f }
      DefaultConfiguration()
  end
end

local function all_files(prefix, basenames)
  local t = {}

  for _,f in pairs(basenames) do
    t[_] = path.join(prefix, f)
  end
  return t
end

-- CHAPITRE 1
solution "Chap1"
  location ( LocationDir )
  configurations { "Debug", "Release" }

  flags { "ExtraWarnings"}
  includedirs { path.join(Root, "1") }
  buildoptions { "-std=c++11" }

  simple_project("Chap1_", os.matchfiles(path.join(Root, "1/*.cc")))

  project ("Chap1_" .. "add_item") files (path.join(Root, "1/Sales_item.h"))
  project ("Chap1_" .. "add_item2") files (path.join(Root, "1/Sales_item.h"))
  project ("Chap1_" .. "avg_price") files (path.join(Root, "1/Sales_item.h"))
  project ("Chap1_" .. "item_io") files (path.join(Root, "1/Sales_item.h"))

-- CHAPITRE 2
solution "Chap2"
  location ( LocationDir )
  configurations { "Debug", "Release" }

  flags { "ExtraWarnings"}
  includedirs { path.join(Root, "1") }
  includedirs { path.join(Root, "2") }
  buildoptions { "-std=c++11" }

  simple_project("Chap2_", os.matchfiles(path.join(Root, "2/*.cc")))

-- CHAPITRE 3
solution "Chap3"
  location ( LocationDir )
  configurations { "Debug", "Release" }

  flags { "ExtraWarnings"}
  includedirs { path.join(Root, "1") }
  buildoptions { "-std=c++11" }

  simple_project("Chap3_", os.matchfiles(path.join(Root, "3/*.cc")))

-- CHAPITRE 4
solution "Chap4"
  location ( LocationDir )
  configurations { "Debug", "Release" }

  flags { "ExtraWarnings"}
  includedirs { path.join(Root, "2") }
  buildoptions { "-std=c++11" }

  simple_project("Chap4_", os.matchfiles(path.join(Root, "4/*.cc")))

-- CHAPITRE 5
solution "Chap5"
  location ( LocationDir )
  configurations { "Debug", "Release" }

  flags { "ExtraWarnings"}
  includedirs { path.join(Root, "1") }
  buildoptions { "-std=c++11" }

  simple_project("Chap5_", os.matchfiles(path.join(Root, "5/*.cc")))

-- CHAPITRE 6
solution "Chap6"
  location ( LocationDir )
  configurations { "Debug", "Release" }

  flags { "ExtraWarnings"}
  buildoptions { "-std=c++11" }

  simple_project("Chap6_", all_files(path.join(Root, "6/"),
    {"arrRet.cc", "count-calls.cc", "errMsg_initList.cc",
      "fact.cc", "fcnptrRet.cc", "good_printmain.cc",
      "inline_shorter.cc", "mainmath.cc", "mainret.cc",
      "mk_plural.cc", "ref-fcn.cc", "refparms.cc", "refret-ex.cc",
      "reset.cc", "stl-arr-fcns.cc", "usefcnptr.cc", "wdebug.cc"
    }))

  project ("Chap6_" .. "fact") files (path.join(Root, "6/LocalMath.*"))
  project ("Chap6_" .. "mainmath") files (path.join(Root, "6/LocalMath.*"))
